package com.ydl.auth.service.utils;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.OSSObjectSummary;
import com.aliyun.oss.model.ObjectListing;
import com.qiniu.util.Auth;
import com.ydl.auth.service.beans.OssProperties;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.DigestUtils;

import java.io.Serializable;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @program ydl-auth
 * @description: OssUtil
 * @author: wangjianfeng
 * @create: 2020/03/03 19:12
 */
@Data
@Component
public class OssUrlUtil implements Serializable {


	@Autowired
	private OssProperties ossProperties;

	public String getAliyTemporaryUrl(String host,String key,Long expressedTime){


		String tagetUrl = "";
		List<String> contentList = getAliyContentList(key);
		// 创建OSSClient实例。
		OSS ossClient = new OSSClientBuilder().build(ossProperties.getAliossEndpoint(),
				ossProperties.getAliossKeyAccessKeyId(), ossProperties.getAliossKeyAccessKeySecret());
		for (String objectName:contentList) {


			Date expiration = new Date(System.currentTimeMillis() + expressedTime);
			// 生成以GET方法访问的签名URL，访客可以直接通过浏览器访问相关内容。
			String replaceStr = ossProperties.getAliossBucketName()+ossProperties.getAliossEndpoint()
																			.replace("http://","")
																			.replace("https://","");
			URL url = ossClient.generatePresignedUrl(ossProperties.getAliossBucketName(), objectName, expiration);
			String tag_url = url.toString().replace(replaceStr,host);
			System.out.println("aliyun 生成的临时url为 "+tag_url);
			tagetUrl = tag_url;
		}


		// 关闭OSSClient。
		ossClient.shutdown();

		return tagetUrl;
	}

	public  String getYpyTemporaryUrl(String host,String key,Long expressedTime){

		Long currentTime = System.currentTimeMillis();
		Long etime = currentTime + expressedTime;
		String urlString = "/"+key;
		String sign = DigestUtils.md5DigestAsHex( (ossProperties.getYpSecret()+ "&"+ etime+"&"+ urlString).getBytes() );
		Integer startIndex = sign.length()/2-4;
		Integer endIndex = sign.length()/2+4;
		String temStr = sign.substring(startIndex, endIndex);
		String _upt = temStr + etime;
		String requestUrl = host +urlString+"?_upt="+_upt;


		return requestUrl;
	}

	public  String getQnyTemporaryUrl(String host,String key,Long expressedTime) {

		// 文档url https://developer.qiniu.com/kodo/sdk/1239/java#private-get
		String finalUrl = "";
		try {
			String accessKey = ossProperties.getQnossKeyAccessKey();
			String secretKey = ossProperties.getQnossKeySecretKey();
			String fileName = key;
			String domainOfBucket = host;
			String encodedFileName = URLEncoder.encode(fileName, "utf-8").replace("+", "%20");
			String publicUrl = String.format("%s/%s", domainOfBucket, encodedFileName);

			Auth auth = Auth.create(accessKey, secretKey);

			finalUrl = auth.privateDownloadUrl(publicUrl, expressedTime);
			System.out.println("七牛云url " +finalUrl);

			return finalUrl;

		} catch (Exception e) {
			e.printStackTrace();
		}

		return finalUrl;

	}

	public  List<String> getAliyContentList(String KeyPrefix){


		List<String> result = new ArrayList<>();
		// 创建OSSClient实例。
		OSS ossClient = new OSSClientBuilder().build(ossProperties.getAliossEndpoint(),
				ossProperties.getAliossKeyAccessKeyId(), ossProperties.getAliossKeyAccessKeySecret());

		// 列举文件。 如果不设置KeyPrefix，则列举存储空间下所有的文件。KeyPrefix，则列举包含指定前缀的文件。
		ObjectListing objectListing = ossClient.listObjects(ossProperties.getAliossBucketName(),KeyPrefix);
		List<OSSObjectSummary> sums = objectListing.getObjectSummaries();
		for (OSSObjectSummary s : sums) {
			result.add(s.getKey());
			System.out.println("\t" + s.getKey());
		}

		// 关闭OSSClient。
		ossClient.shutdown();

		return result;

	}




}
